#include "resumptiontoken.h"
#include "oaitools.h"

/**
 *  Generate an initial resumption token from some basic details.
 *
 *  TODO: add optional argument to set the server name.
 */
ResumptionToken::ResumptionToken(const text_t &collection, const text_t &node,
				 const text_t &buildDate)
{ this->collection = collection;
  this->browseNode = node;
  this->buildDate  = buildDate;
  this->startItem  = 0;
}

/**
 *  Generate a resumption token from a URN-style format.
 *
 *  See getToken() for details of the format.
 *
 *  TODO: support inclusion of an optional server name.
 */
ResumptionToken::ResumptionToken(const text_t &URN)
{ text_t::const_iterator first = URN.begin();
  text_t::const_iterator last = URN.end(); 
  text_t::const_iterator second;

  this->collection = "";
  this->browseNode = "";
  this->startItem  = -1;
  
  text_t::const_iterator here = findchar(first, last, ':');
  if (here == first) {
    return;
  }
  
  text_t oainamespace = substr(first, here);
  if (oainamespace != "gsdloai") {
    return;
  }

  // increment past the first colon to get the location
  first = ++here;  

  // get the collection, browseNode
  here = findchar(first, last, ',');
  if (here == last) {
    return;
  }

  second = findchar(first, here,'.');
  this->collection = substr(first, second);

  //  cerr << "Collection " << this->collection << endl;

  if (second != here) {
    this->browseNode = substr(second, here);
  }
  else {
    first = here;
  }
  // get past the ','
  first = ++here;

  // find the second ',' to delimit the position stack
  second = findchar(first, last, ',');

  // if not found, then get build and start item
  if (second != first) {
    // extract list and step past it
    text_t offsetList = substr(first, second);
    first = ++second;
    
    do {
      second = findchar(offsetList.begin(), offsetList.end(), '.');
      if (second == offsetList.end()) 
	break;
      
      // extract and push the next position
      text_t thisPos = substr(offsetList.begin(), second);
      this->browsePosition.push_back(thisPos.getint());

      // pop the position from the list
      offsetList = substr(++second, offsetList.end());
    } while (true);    
    this->browsePosition.push_back(offsetList.getint());
  }
  else {
    first ++;
  }

  // now find the build date marker
  here = findchar(first, last, '-');
  if (here == first) {
    this->startItem = substr(first, last).getint();
  }
  else {
    this->startItem = substr(first, here).getint();
    this->buildDate = substr(++here, last); 
  }
}

/**
 *  Get a resumption token in text_t format.
 * 
 *  Resumption tokens are in the format:
 *
 *    gsdloai:<serverName>:collectionname.browseNode,startItem-BuildDate
 *
 *  The resumption token format does not currently implement the use of
 *  the optional <serverName> item; it is taken to default to the name of
 *  the receiving server.
 *
 *  TODO: add server identity as an optional argument; also change
 *        ResumptionToken(text_t &) accordingly.
 */
text_t ResumptionToken::getToken()
{ text_t reply = "gsdloai:";
  reply = reply + this->collection;
  if (this->browseNode != "") {
    reply = reply + "." + this->browseNode;
  }
  reply = reply + ",";
  for (int i = 0; i < this->browsePosition.size(); i++) {
    if (i != 0) {
      reply.append(".");
    }
    reply.appendint(i);
  }
  reply = reply + ",";
  reply.append(this->startItem);
  reply = reply + "-" + buildDate;

  return reply;
}

/**
 *  Update the position of an existing resumption token
 */
void ResumptionToken::setPosition(const text_t &node, int startItem)
{ this->browseNode = node;
  this->startItem  = startItem; 
}

/**
 *  Check if the resumption token is valid - only a very primitive
 *  check is done here; one ought to check for an existing collection
 *  and valid browse Node, build date and startItem
 *
 *  TODO: implement improved validation checking.
 */
bool ResumptionToken::isValid()
{ return this->collection != "";
}
